@@ -1,9 +1,9 @@
-/* $Id: fxmesa.h,v 3.1 1998/04/01 03:00:28 brianp Exp $ */
+/* $Id: fxmesa.h,v 1.3 2001/06/20 19:52:11 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.0
- * Copyright (C) 1995-1998  Brian Paul
+ * Version:  3.3
+ * Copyright (C) 1995-2001  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,17 +21,6 @@
  */
 
 
-/*
- * $Log: fxmesa.h,v $
- * Revision 3.1  1998/04/01 03:00:28  brianp
- * updated for v0.24 of 3Dfx/Glide driver
- *
- * Revision 3.0  1998/02/20 05:04:45  brianp
- * initial rev
- *
- */
-
-
 /*
  * FXMesa - 3Dfx Glide driver for Mesa.  Contributed by David Bucciarelli
  *
@@ -52,7 +41,7 @@ extern "C" {
 
 
 #define FXMESA_MAJOR_VERSION 3
-#define FXMESA_MINOR_VERSION 0
+#define FXMESA_MINOR_VERSION 5
 
 
 /*
@@ -75,30 +64,30 @@ typedef struct tfxMesaContext *fxMesaContext;
 #endif
 
 
-WINGDIAPI fxMesaContext APIENTRY fxMesaCreateContext(GLuint win, GrScreenResolution_t,
+GLAPI fxMesaContext GLAPIENTRY fxMesaCreateContext(GLuint win, GrScreenResolution_t,
 						  GrScreenRefresh_t,
 						  const GLint attribList[]);
 
-WINGDIAPI fxMesaContext APIENTRY fxMesaCreateBestContext(GLuint win,
+GLAPI fxMesaContext GLAPIENTRY fxMesaCreateBestContext(GLuint win,
 						      GLint width, GLint height,
 						      const GLint attribList[]);
-WINGDIAPI void APIENTRY fxMesaDestroyContext(fxMesaContext ctx);
+GLAPI void GLAPIENTRY fxMesaDestroyContext(fxMesaContext ctx);
 
-WINGDIAPI GLboolean APIENTRY fxMesaSelectCurrentBoard(int n);
+GLAPI GLboolean GLAPIENTRY fxMesaSelectCurrentBoard(int n);
 
-WINGDIAPI void APIENTRY fxMesaMakeCurrent(fxMesaContext ctx);
+GLAPI void GLAPIENTRY fxMesaMakeCurrent(fxMesaContext ctx);
 
-WINGDIAPI fxMesaContext APIENTRY fxMesaGetCurrentContext(void);
+GLAPI fxMesaContext GLAPIENTRY fxMesaGetCurrentContext(void);
 
-WINGDIAPI void APIENTRY fxMesaSwapBuffers(void);
+GLAPI void GLAPIENTRY fxMesaSwapBuffers(void);
 
-WINGDIAPI void APIENTRY fxMesaSetNearFar(GLfloat nearVal, GLfloat farVal);
+GLAPI void GLAPIENTRY fxMesaSetNearFar(GLfloat nearVal, GLfloat farVal);
 
-WINGDIAPI void APIENTRY fxMesaUpdateScreenSize(fxMesaContext ctx);
+GLAPI void GLAPIENTRY fxMesaUpdateScreenSize(fxMesaContext ctx);
 
-WINGDIAPI int APIENTRY fxQueryHardware(void);
+GLAPI int GLAPIENTRY fxQueryHardware(void);
 
-WINGDIAPI void APIENTRY fxCloseHardware(void);
+GLAPI void GLAPIENTRY fxCloseHardware(void);
 
 
 #if defined (__BEOS__)
