@@ -1,49 +1,27 @@
-/* $Id: osmesa.h,v 1.7 1997/09/29 23:22:16 brianp Exp $ */
+/* $Id: osmesa.h,v 1.6 2000/09/08 16:41:38 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-
-/*
- * $Log: osmesa.h,v $
- * Revision 1.7  1997/09/29 23:22:16  brianp
- * updated version to 2.5
- *
- * Revision 1.6  1997/07/11 23:07:16  brianp
- * added OSMesaGetDepthBuffer function (Randy Frank)
- *
- * Revision 1.5  1997/02/19 10:13:54  brianp
- * now test for __QUICKDRAW__ like for __BEOS__ (Randy Frank)
- *
- * Revision 1.4  1997/02/10 20:31:41  brianp
- * added OSMESA_RGB and OSMESA_BGR, per Randy Frank
- *
- * Revision 1.3  1997/02/03 20:07:40  brianp
- * patches for BeOS
- *
- * Revision 1.2  1996/10/30 03:12:30  brianp
- * incremented version to 2.1
- *
- * Revision 1.1  1996/09/13 01:26:41  brianp
- * Initial revision
- *
+ * Version:  3.5
+ * 
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
@@ -68,26 +46,23 @@
  * The limits on the width and height of an image buffer are MAX_WIDTH and
  * MAX_HEIGHT as defined in Mesa/src/config.h.  Defaults are 1280 and 1024.
  * You can increase them as needed but beware that many temporary arrays in
- * Mesa are dimensioned by MAX_WIDTH or MAX_WIDTH.
+ * Mesa are dimensioned by MAX_WIDTH or MAX_HEIGHT.
  */
 
 
-
 #ifndef OSMESA_H
 #define OSMESA_H
 
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 
-#include "GL/gl.h"
+#include <GL/gl.h>
 
 
-
-#define OSMESA_MAJOR_VERSION 2
+#define OSMESA_MAJOR_VERSION 3
 #define OSMESA_MINOR_VERSION 5
 
 
@@ -121,7 +96,6 @@ extern "C" {
 #define OSMESA_TYPE		0x23
 
 
-
 typedef struct osmesa_context *OSMesaContext;
 
 
@@ -140,18 +114,31 @@ typedef struct osmesa_context *OSMesaContext;
  *                     display lists.  NULL indicates no sharing.
  * Return:  an OSMesaContext or 0 if error
  */
-extern OSMesaContext OSMesaCreateContext( GLenum format,
-                                          OSMesaContext sharelist );
+GLAPI OSMesaContext GLAPIENTRY
+OSMesaCreateContext( GLenum format, OSMesaContext sharelist );
 
 
 
+/*
+ * Create an Off-Screen Mesa rendering context and specify desired
+ * size of depth buffer, stencil buffer and accumulation buffer.
+ * If you specify zero for depthBits, stencilBits, accumBits you
+ * can save some memory.
+ *
+ * New in Mesa 3.5
+ */
+GLAPI OSMesaContext GLAPIENTRY
+OSMesaCreateContextExt( GLenum format, GLint depthBits, GLint stencilBits,
+                        GLint accumBits, OSMesaContext sharelist);
+
 
 /*
  * Destroy an Off-Screen Mesa rendering context.
  *
  * Input:  ctx - the context to destroy
  */
-extern void OSMesaDestroyContext( OSMesaContext ctx );
+GLAPI void GLAPIENTRY
+OSMesaDestroyContext( OSMesaContext ctx );
 
 
 
@@ -182,9 +169,9 @@ extern void OSMesaDestroyContext( OSMesaContext ctx );
  *          invalid buffer address, type!=GL_UNSIGNED_BYTE, width<1, height<1,
  *          width>internal limit or height>internal limit.
  */
-extern GLboolean OSMesaMakeCurrent( OSMesaContext ctx,
-                                    void *buffer, GLenum type,
-                                    GLsizei width, GLsizei height );
+GLAPI GLboolean GLAPIENTRY
+OSMesaMakeCurrent( OSMesaContext ctx, void *buffer, GLenum type,
+                   GLsizei width, GLsizei height );
 
 
 
@@ -192,7 +179,8 @@ extern GLboolean OSMesaMakeCurrent( OSMesaContext ctx,
 /*
  * Return the current Off-Screen Mesa rendering context handle.
  */
-extern OSMesaContext OSMesaGetCurrentContext( void );
+GLAPI OSMesaContext GLAPIENTRY
+OSMesaGetCurrentContext( void );
 
 
 
@@ -209,12 +197,13 @@ extern OSMesaContext OSMesaGetCurrentContext( void );
  *
  * New in version 2.0.
  */
-extern void OSMesaPixelStore( GLint pname, GLint value );
+GLAPI void GLAPIENTRY
+OSMesaPixelStore( GLint pname, GLint value );
 
 
 
 /*
- * Return context info.  This is like glGetIntegerv.
+ * Return an integer value like glGetIntegerv.
  * Input:  pname -
  *                 OSMESA_WIDTH  return current image width
  *                 OSMESA_HEIGHT  return current image height
@@ -224,7 +213,8 @@ extern void OSMesaPixelStore( GLint pname, GLint value );
  *                 OSMESA_Y_UP returns 1 or 0 to indicate Y axis direction
  *         value - pointer to integer in which to return result.
  */
-extern void OSMesaGetIntegerv( GLint pname, GLint *value );
+GLAPI void GLAPIENTRY
+OSMesaGetIntegerv( GLint pname, GLint *value );
 
 
 
@@ -238,12 +228,27 @@ extern void OSMesaGetIntegerv( GLint pname, GLint *value );
  *
  * New in Mesa 2.4.
  */
-extern GLboolean OSMesaGetDepthBuffer( OSMesaContext c,
-                                       GLint *width, GLint *height,
-                                       GLint *bytesPerValue, void **buffer );
+GLAPI GLboolean GLAPIENTRY
+OSMesaGetDepthBuffer( OSMesaContext c, GLint *width, GLint *height,
+                      GLint *bytesPerValue, void **buffer );
 
 
 
+/*
+ * Return the color buffer associated with an OSMesa context.
+ * Input:  c - the OSMesa context
+ * Output:  width, height - size of buffer in pixels
+ *          format - buffer format (OSMESA_FORMAT)
+ *          buffer - pointer to depth buffer values
+ * Return:  GL_TRUE or GL_FALSE to indicate success or failure.
+ *
+ * New in Mesa 3.3.
+ */
+GLAPI GLboolean GLAPIENTRY
+OSMesaGetColorBuffer( OSMesaContext c, GLint *width, GLint *height,
+                      GLint *format, void **buffer );
+
+
 
 #if defined(__BEOS__) || defined(__QUICKDRAW__)
 #pragma export off
