@@ -1,8 +1,8 @@
-/* $Id: polytest.c,v 1.2.2.1 2000/07/11 02:07:52 brianp Exp $ */
+/* $Id: polytest.c,v 1.4 2000/11/10 17:23:04 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.2.1
+ * Version:  3.3
  * Copyright (C) 1995-2000  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -786,7 +786,7 @@ cut_out_hole(GLUtriangulatorObj * tobj,
    tess_vertex *v1, *v2, *tmp_vertex;
    GLuint vertex1_cnt, vertex2_cnt, tmp_vertex_cnt;
    GLuint i, j, k;
-   GLenum test;
+   GLenum test = 0;
 
    /* find an edge connecting contour and hole not intersecting any other */
    /* edge belonging to either the contour or any of the other holes */
