@@ -27,6 +27,13 @@
 #include <ggi/internal/ggi-dl.h>
 #include <ggi/mesa/ggimesa_int.h>
 
+#include "mmath.h"
+#include "swrast/swrast.h"
+#include "swrast_setup/swrast_setup.h"
+#include "swrast/s_context.h"
+#include "swrast/s_depth.h"
+#include "swrast/s_triangle.h"
+
 /**********************************************************************/
 /*****            Write spans of pixels                           *****/
 /**********************************************************************/
@@ -291,65 +298,66 @@ void GGIread_rgba_pixels( const GLcontext *ctx,
 }
 
 
-triangle_func ggiGetTriangleFunc(GLcontext *ctx);
+static swrast_tri_func ggimesa_stubs_get_triangle_func(GLcontext *ctx);
 
-int GGIsetup_driver(GGIMesaContext ggictx,struct ggi_mesa_info *info)
+int GGIsetup_driver(GGIMesaContext ggictx, struct ggi_mesa_info *info)
 {
-	GLcontext *ctx=ggictx->gl_ctx;
+	GLcontext *ctx = ggictx->gl_ctx;
 
 	ctx->Driver.WriteRGBASpan	= GGIwrite_rgba_span;
 	ctx->Driver.WriteRGBSpan	= GGIwrite_rgb_span;
-	ctx->Driver.WriteMonoRGBASpan   = GGIwrite_mono_span;
 	ctx->Driver.WriteRGBAPixels     = GGIwrite_rgba_pixels;
-	ctx->Driver.WriteMonoRGBAPixels = GGIwrite_mono_pixels;
 
 	ctx->Driver.WriteCI32Span       = GGIwrite_ci32_span;
 	ctx->Driver.WriteCI8Span       	= GGIwrite_ci8_span;
-	ctx->Driver.WriteMonoCISpan   	= GGIwrite_mono_span;
 	ctx->Driver.WriteCI32Pixels     = GGIwrite_ci32_pixels;
-	ctx->Driver.WriteMonoCIPixels 	= GGIwrite_mono_pixels;
 
 	ctx->Driver.ReadCI32Span 	= GGIread_ci32_span;
 	ctx->Driver.ReadRGBASpan	= GGIread_rgba_span;
 	ctx->Driver.ReadCI32Pixels	= GGIread_ci32_pixels;
 	ctx->Driver.ReadRGBAPixels	= GGIread_rgba_pixels;
-
+	
 	return 0;
 }
 
 void GGIupdate_state(GLcontext *ctx)
 {
-	ctx->Driver.TriangleFunc = ggiGetTriangleFunc(ctx);
+	ctx->Driver.TriangleFunc = _swsetup_Triangle;
 }
 
 
-void GGItriangle_flat(GLcontext *ctx,GLuint v0,GLuint v1,GLuint v2,GLuint pv)
+void GGItriangle_flat(GLcontext *ctx, const SWvertex *v0, const SWvertex *v1, const SWvertex *v2)
 {
-#define SETUP_CODE			\
-	GLubyte r = VB->ColorPtr->data[pv][0];	\
-	GLubyte g = VB->ColorPtr->data[pv][1];	\
-	GLubyte b = VB->ColorPtr->data[pv][2];	\
-	GLubyte a = VB->ColorPtr->data[pv][3];	\
-	(*ctx->Driver.Color)(ctx,r,g,b,a);
+//#define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE	  
+	
+#define SETUP_CODE	       	\
+	ggi_color color;	\
+	color.r = v0->color[0];	\
+	color.g = v0->color[1];	\
+	color.b = v0->color[2];	\
+	color.a = v0->color[3];	\
+	ggiSetGCForeground(VIS, ggiMapColor(VIS, &color));
 
 #define INNER_LOOP(LEFT,RIGHT,Y)				\
 		ggiDrawHLine(VIS,LEFT,FLIP(Y),RIGHT-LEFT);	
 		
-#include "tritemp.h"	
+#include "swrast/s_tritemp.h"
 }
 
 
-void GGItriangle_flat_depth(GLcontext *ctx,GLuint v0,GLuint v1,
-			    GLuint v2,GLuint pv)
+static void GGItriangle_flat_depth(GLcontext *ctx, const SWvertex *v0, const SWvertex *v1, const SWvertex *v2)
 {
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 
-#define SETUP_CODE			\
-	GLubyte r = VB->ColorPtr->data[pv][0];	\
-	GLubyte g = VB->ColorPtr->data[pv][1];	\
-	GLubyte b = VB->ColorPtr->data[pv][2];	\
-	GLubyte a = VB->ColorPtr->data[pv][3];	\
-	(*ctx->Driver.Color)(ctx,r,g,b,a);
+#define SETUP_CODE	       	\
+	ggi_color color;	\
+	color.r = v0->color[0];	\
+	color.g = v0->color[1];	\
+	color.b = v0->color[2];	\
+	color.a = v0->color[3];	\
+	ggiSetGCForeground(VIS, ggiMapColor(VIS, &color));
 	
 #define INNER_LOOP(LEFT,RIGHT,Y)					\
 	{								\
@@ -376,16 +384,16 @@ void GGItriangle_flat_depth(GLcontext *ctx,GLuint v0,GLuint v1,
 	if (length) ggiDrawHLine(VIS,startx,yy,length);			\
 	}
 
-#include "tritemp.h"	
+#include "swrast/s_tritemp.h"
 }
 
 
-triangle_func ggiGetTriangleFunc(GLcontext *ctx)
+static swrast_tri_func ggimesa_stubs_get_triangle_func(GLcontext *ctx)
 {
 	if (ctx->Stencil.Enabled) return NULL;
 	if (ctx->Polygon.SmoothFlag) return NULL;
 	if (ctx->Polygon.StippleFlag) return NULL;
-	if (ctx->Texture.Enabled) return NULL;  
+	if (ctx->Texture._ReallyEnabled) return NULL;  
 	if (ctx->Light.ShadeModel==GL_SMOOTH) return NULL;
 	if (ctx->Depth.Test && ctx->Depth.Func != GL_LESS) return NULL;
 
@@ -395,15 +403,28 @@ triangle_func ggiGetTriangleFunc(GLcontext *ctx)
 	return GGItriangle_flat;	
 }
 
-int GGIdlinit(ggi_visual_t vis, const char *version, void *argptr)
+static int GGIopen(ggi_visual_t vis, struct ggi_dlhandle *dlh,
+			const char *args, void *argptr, uint32 *dlret)
 { 
        	LIBGGI_MESAEXT(vis)->update_state = GGIupdate_state;
 	LIBGGI_MESAEXT(vis)->setup_driver = GGIsetup_driver;
 
+	*dlret = GGI_DL_OPDRAW;
 	return 0;
 }
 
-int GGIdlcleanup(ggi_visual_t vis)
+int MesaGGIdl_stubs(int func, void **funcptr)
 {
-	return 0;
+	switch (func) {
+		case GGIFUNC_open:
+			*funcptr = GGIopen;
+			return 0;
+		case GGIFUNC_exit:
+		case GGIFUNC_close:
+			*funcptr = NULL;
+			return 0;
+		default:
+			*funcptr = NULL;
+	}
+	return GGI_ENOTFOUND;
 }
