@@ -1,4 +1,4 @@
-/* $Id: wgl.c,v 1.3 1998/09/16 02:46:56 brianp Exp $ */
+/* $Id: wgl.c,v 1.6 2001/05/09 20:45:00 brianp Exp $ */
 
 /*
 * This library is free software; you can redistribute it and/or
@@ -23,32 +23,27 @@
 * Some things originated from the 3Dfx WGL functions
 */
 
-/*
- * $Log: wgl.c,v $
- * Revision 1.3  1998/09/16 02:46:56  brianp
- * added RCS Id and Log tokens
- *
- */
-
 #ifdef WIN32
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include <windows.h>
+#define GL_GLEXT_PROTOTYPES
 #include <GL/gl.h>
-#include <GL/glu.h>
+#include <GL/glext.h>
+//#include <GL/glu.h>
 
 #ifdef __cplusplus
 }
 #endif
 
 #include <stdio.h>
-#include <windows.h>
 #include <tchar.h>
 #include "wmesadef.h"
 #include "GL/wmesa.h"
-#include "types.h"
+#include "mtypes.h"
 
 #define MAX_MESA_ATTRS	20
 
@@ -67,31 +62,41 @@ struct __pixelformat__
 struct __extensions__	ext[] = {
 
 #ifdef GL_EXT_polygon_offset
-    {	(PROC)glPolygonOffsetEXT,		"glPolygonOffsetEXT"			},
+   { (PROC)glPolygonOffsetEXT,			"glPolygonOffsetEXT"		},
 #endif
-    {	(PROC)glBlendEquationEXT,		"glBlendEquationEXT"			},
-    {	(PROC)glBlendColorEXT,			"glBlendColorExt"			},
-    {	(PROC)glVertexPointerEXT,		"glVertexPointerEXT"			},
-    {	(PROC)glNormalPointerEXT,		"glNormalPointerEXT"			},
-    {	(PROC)glColorPointerEXT,		"glColorPointerEXT"			},
-    {	(PROC)glIndexPointerEXT,		"glIndexPointerEXT"			},
-    {	(PROC)glTexCoordPointerEXT,		"glTexCoordPointer"			},
-    {	(PROC)glEdgeFlagPointerEXT,		"glEdgeFlagPointerEXT"		        },
-    {	(PROC)glGetPointervEXT,			"glGetPointervEXT"			},
-    {	(PROC)glArrayElementEXT,		"glArrayElementEXT"			},
-    {	(PROC)glDrawArraysEXT,			"glDrawArrayEXT"			},
-    {	(PROC)glAreTexturesResidentEXT,	"glAreTexturesResidentEXT"	                },
-    {	(PROC)glBindTextureEXT,			"glBindTextureEXT"			},
-    {	(PROC)glDeleteTexturesEXT,		"glDeleteTexturesEXT"			},
-    {	(PROC)glGenTexturesEXT,			"glGenTexturesEXT"			},
-    {	(PROC)glIsTextureEXT,			"glIsTextureEXT"			},
-    {	(PROC)glPrioritizeTexturesEXT,	"glPrioritizeTexturesEXT"		        },
-    {	(PROC)glCopyTexSubImage3DEXT,	"glCopyTexSubImage3DEXT"		        },
-    {	(PROC)glTexImage3DEXT,			"glTexImage3DEXT"			},
-    {	(PROC)glTexSubImage3DEXT,		"glTexSubImage3DEXT"			},
+   { (PROC)glBlendEquationEXT,			"glBlendEquationEXT"		},
+   { (PROC)glBlendColorEXT,			"glBlendColorExt"		},
+   { (PROC)glVertexPointerEXT,			"glVertexPointerEXT"		},
+   { (PROC)glNormalPointerEXT,			"glNormalPointerEXT"		},
+   { (PROC)glColorPointerEXT,			"glColorPointerEXT"		},
+   { (PROC)glIndexPointerEXT,			"glIndexPointerEXT"		},
+   { (PROC)glTexCoordPointerEXT,		"glTexCoordPointer"		},
+   { (PROC)glEdgeFlagPointerEXT,		"glEdgeFlagPointerEXT"		},
+   { (PROC)glGetPointervEXT,			"glGetPointervEXT"		},
+   { (PROC)glArrayElementEXT,			"glArrayElementEXT"		},
+   { (PROC)glDrawArraysEXT,			"glDrawArrayEXT"		},
+   { (PROC)glAreTexturesResidentEXT,		"glAreTexturesResidentEXT"	},
+   { (PROC)glBindTextureEXT,			"glBindTextureEXT"		},
+   { (PROC)glDeleteTexturesEXT,			"glDeleteTexturesEXT"		},
+   { (PROC)glGenTexturesEXT,			"glGenTexturesEXT"		},
+   { (PROC)glIsTextureEXT,			"glIsTextureEXT"		},
+   { (PROC)glPrioritizeTexturesEXT,		"glPrioritizeTexturesEXT"	},
+   { (PROC)glCopyTexSubImage3DEXT,		"glCopyTexSubImage3DEXT"	},
+   { (PROC)glTexImage3DEXT,			"glTexImage3DEXT"		},
+   { (PROC)glTexSubImage3DEXT,			"glTexSubImage3DEXT"		},
+   { (PROC)glColorTableEXT,			"glColorTableEXT"		},
+   { (PROC)glColorSubTableEXT,			"glColorSubTableEXT"		},
+   { (PROC)glGetColorTableEXT,			"glGetColorTableEXT"		},
+   { (PROC)glGetColorTableParameterfvEXT,	"glGetColorTableParameterfvEXT"	},
+   { (PROC)glGetColorTableParameterivEXT,	"glGetColorTableParameterivEXT"	},
+   { (PROC)glPointParameterfEXT,		"glPointParameterfEXT"		},
+   { (PROC)glPointParameterfvEXT,		"glPointParameterfvEXT"		},
+   { (PROC)glBlendFuncSeparateEXT,		"glBlendFuncSeparateEXT"	},
+   { (PROC)glLockArraysEXT,			"glLockArraysEXT"		},
+   { (PROC)glUnlockArraysEXT,			"glUnlockArraysEXT"		}
 };
 
-int				qt_ext = sizeof(ext) / sizeof(ext[0]);
+int qt_ext = sizeof(ext) / sizeof(ext[0]);
 
 struct __pixelformat__	pix[] =
 {
@@ -123,15 +128,15 @@ typedef struct {
 static MesaWglCtx wgl_ctx[MESAWGL_CTX_MAX_COUNT];
 
 static unsigned ctx_count = 0;
-static unsigned ctx_current = 0;
+static unsigned ctx_current = -1;
 static unsigned curPFD = 0;
 
-WINGDIAPI BOOL WINAPI wglCopyContext(HGLRC hglrcSrc,HGLRC hglrcDst,UINT mask)
+GLAPI BOOL GLWINAPI wglCopyContext(HGLRC hglrcSrc,HGLRC hglrcDst,UINT mask)
 {
     return(FALSE);
 }
 
-WINGDIAPI HGLRC WINAPI wglCreateContext(HDC hdc)
+GLAPI HGLRC GLWINAPI wglCreateContext(HDC hdc)
 {
     HWND		hWnd;
     int i = 0;
@@ -165,14 +170,14 @@ WINGDIAPI HGLRC WINAPI wglCreateContext(HDC hdc)
     return(NULL);
 }
 
-WINGDIAPI BOOL WINAPI wglDeleteContext(HGLRC hglrc)
+GLAPI BOOL GLWINAPI wglDeleteContext(HGLRC hglrc)
 {
     int i;
     for ( i = 0; i < MESAWGL_CTX_MAX_COUNT; i++ )
     {
-    	if ( wgl_ctx[i].ctx == hglrc )
+    	if ( wgl_ctx[i].ctx == (PWMC) hglrc )
     	{
-            WMesaMakeCurrent(hglrc);
+            WMesaMakeCurrent((PWMC) hglrc);
             WMesaDestroyContext();
             wgl_ctx[i].ctx = NULL;
             wgl_ctx[i].hdc = NULL;
@@ -184,21 +189,21 @@ WINGDIAPI BOOL WINAPI wglDeleteContext(HGLRC hglrc)
     return(FALSE);
 }
 
-WINGDIAPI HGLRC WINAPI wglCreateLayerContext(HDC hdc,int iLayerPlane)
+GLAPI HGLRC GLWINAPI wglCreateLayerContext(HDC hdc,int iLayerPlane)
 {
     SetLastError(0);
     return(NULL);
 }
 
-WINGDIAPI HGLRC WINAPI wglGetCurrentContext(VOID)
+GLAPI HGLRC GLWINAPI wglGetCurrentContext(VOID)
 {
    if (ctx_current < 0)
       return 0;
    else
-      return wgl_ctx[ctx_current].ctx;
+      return (HGLRC) wgl_ctx[ctx_current].ctx;
 }
 
-WINGDIAPI HDC WINAPI wglGetCurrentDC(VOID)
+GLAPI HDC GLWINAPI wglGetCurrentDC(VOID)
 {
    if (ctx_current < 0)
       return 0;
@@ -206,7 +211,7 @@ WINGDIAPI HDC WINAPI wglGetCurrentDC(VOID)
       return wgl_ctx[ctx_current].hdc;
 }
 
-WINGDIAPI BOOL WINAPI wglMakeCurrent(HDC hdc,HGLRC hglrc)
+GLAPI BOOL GLWINAPI wglMakeCurrent(HDC hdc,HGLRC hglrc)
 {
     int i;
 
@@ -219,7 +224,7 @@ WINGDIAPI BOOL WINAPI wglMakeCurrent(HDC hdc,HGLRC hglrc)
 
     for ( i = 0; i < MESAWGL_CTX_MAX_COUNT; i++ )
     {
-        if ( wgl_ctx[i].ctx == hglrc )
+        if ( wgl_ctx[i].ctx == (PWMC) hglrc )
         {
             wgl_ctx[i].hdc = hdc;
             WMesaMakeCurrent( (WMesaContext) hglrc );
@@ -230,22 +235,104 @@ WINGDIAPI BOOL WINAPI wglMakeCurrent(HDC hdc,HGLRC hglrc)
     return FALSE;
 }
 
-WINGDIAPI BOOL WINAPI wglShareLists(HGLRC hglrc1,HGLRC hglrc2)
+GLAPI BOOL GLWINAPI wglShareLists(HGLRC hglrc1,HGLRC hglrc2)
 {
     return(TRUE);
 }
 
-WINGDIAPI BOOL WINAPI wglUseFontBitmapsA(HDC hdc,DWORD first,DWORD count,DWORD listBase)
+
+static FIXED FixedFromDouble(double d)
 {
-    return(FALSE);
+   long l = (long) (d * 65536L);
+   return *(FIXED *)&l;
 }
 
-WINGDIAPI BOOL WINAPI wglUseFontBitmapsW(HDC hdc,DWORD first,DWORD count,DWORD listBase)
+GLAPI BOOL GLWINAPI wglUseFontBitmapsA(HDC hdc, DWORD first,
+                                       DWORD count, DWORD listBase)
 {
-    return(FALSE);
+   int i;
+   GLuint font_list;
+   DWORD size;
+   GLYPHMETRICS gm;
+   HANDLE hBits;
+   LPSTR lpBits;
+   MAT2 mat;
+
+   if (first<0)
+      return FALSE;
+   if (count<0)
+      return FALSE;
+   if (listBase<0)
+      return FALSE;
+
+   font_list = listBase;
+
+   mat.eM11 = FixedFromDouble(1);
+   mat.eM12 = FixedFromDouble(0);
+   mat.eM21 = FixedFromDouble(0);
+   mat.eM22 = FixedFromDouble(-1);
+
+   memset(&gm,0,sizeof(gm));
+
+   for (i = 0; i < count; i++)
+   {
+       DWORD err;
+       
+      glNewList( font_list+i, GL_COMPILE );
+
+      /* allocate space for the bitmap/outline */
+      size = GetGlyphOutline(hdc, first + i, GGO_BITMAP, &gm, 0, NULL, &mat);
+      if (size == GDI_ERROR)
+      {
+         err = GetLastError();
+         return(FALSE);
+      }
+
+      hBits  = GlobalAlloc(GHND, size+1);
+      lpBits = GlobalLock(hBits);
+
+      err = 
+      GetGlyphOutline(hdc,    /* handle to device context */
+                      first + i,          /* character to query */
+                      GGO_BITMAP,         /* format of data to return */
+                      &gm,                /* pointer to structure for metrics*/
+                      size,               /* size of buffer for data */
+                      lpBits,             /* pointer to buffer for data */
+                      &mat                /* pointer to transformation */
+                                          /* matrix structure */
+                  );
+
+      if (err == GDI_ERROR)
+      {
+         err = GetLastError();
+
+         GlobalUnlock(hBits);
+         GlobalFree(hBits);
+
+         return(FALSE);
+      }
+
+      glBitmap(gm.gmBlackBoxX,gm.gmBlackBoxY,
+               -gm.gmptGlyphOrigin.x,
+               gm.gmptGlyphOrigin.y,
+               gm.gmCellIncX,gm.gmCellIncY,
+               (const GLubyte * )lpBits);
+
+      GlobalUnlock(hBits);
+      GlobalFree(hBits);
+
+      glEndList( );
+   }
+
+   return TRUE;
 }
 
-WINGDIAPI BOOL WINAPI wglUseFontOutlinesA(HDC hdc,DWORD first,DWORD count,
+GLAPI BOOL GLWINAPI wglUseFontBitmapsW(HDC hdc,DWORD first,DWORD count,DWORD listBase)
+{
+    return FALSE;
+}
+
+GLAPI BOOL GLWINAPI wglUseFontOutlinesA(HDC hdc,DWORD first,DWORD count,
                                   DWORD listBase,FLOAT deviation,
                                   FLOAT extrusion,int format,
                                   LPGLYPHMETRICSFLOAT lpgmf)
@@ -254,7 +341,7 @@ WINGDIAPI BOOL WINAPI wglUseFontOutlinesA(HDC hdc,DWORD first,DWORD count,
     return(FALSE);
 }
 
-WINGDIAPI BOOL WINAPI wglUseFontOutlinesW(HDC hdc,DWORD first,DWORD count,
+GLAPI BOOL GLWINAPI wglUseFontOutlinesW(HDC hdc,DWORD first,DWORD count,
                                   DWORD listBase,FLOAT deviation,
                                   FLOAT extrusion,int format,
                                   LPGLYPHMETRICSFLOAT lpgmf)
@@ -263,7 +350,7 @@ WINGDIAPI BOOL WINAPI wglUseFontOutlinesW(HDC hdc,DWORD first,DWORD count,
     return(FALSE);
 }
 
-WINGDIAPI BOOL WINAPI wglDescribeLayerPlane(HDC hdc,int iPixelFormat,
+GLAPI BOOL GLWINAPI wglDescribeLayerPlane(HDC hdc,int iPixelFormat,
                                     int iLayerPlane,UINT nBytes,
                                     LPLAYERPLANEDESCRIPTOR plpd)
 {
@@ -271,7 +358,7 @@ WINGDIAPI BOOL WINAPI wglDescribeLayerPlane(HDC hdc,int iPixelFormat,
     return(FALSE);
 }
 
-WINGDIAPI int WINAPI wglSetLayerPaletteEntries(HDC hdc,int iLayerPlane,
+GLAPI int GLWINAPI wglSetLayerPaletteEntries(HDC hdc,int iLayerPlane,
                                        int iStart,int cEntries,
                                        CONST COLORREF *pcr)
 {
@@ -279,7 +366,7 @@ WINGDIAPI int WINAPI wglSetLayerPaletteEntries(HDC hdc,int iLayerPlane,
     return(0);
 }
 
-WINGDIAPI int WINAPI wglGetLayerPaletteEntries(HDC hdc,int iLayerPlane,
+GLAPI int GLWINAPI wglGetLayerPaletteEntries(HDC hdc,int iLayerPlane,
                                        int iStart,int cEntries,
                                        COLORREF *pcr)
 {
@@ -287,13 +374,13 @@ WINGDIAPI int WINAPI wglGetLayerPaletteEntries(HDC hdc,int iLayerPlane,
     return(0);
 }
 
-WINGDIAPI BOOL WINAPI wglRealizeLayerPalette(HDC hdc,int iLayerPlane,BOOL bRealize)
+GLAPI BOOL GLWINAPI wglRealizeLayerPalette(HDC hdc,int iLayerPlane,BOOL bRealize)
 {
     SetLastError(0);
     return(FALSE);
 }
 
-WINGDIAPI BOOL WINAPI wglSwapLayerBuffers(HDC hdc,UINT fuPlanes)
+GLAPI BOOL GLWINAPI wglSwapLayerBuffers(HDC hdc,UINT fuPlanes)
 {
     if( !hdc )
     {
@@ -304,7 +391,7 @@ WINGDIAPI BOOL WINAPI wglSwapLayerBuffers(HDC hdc,UINT fuPlanes)
     return(FALSE);
 }
 
-WINGDIAPI int WINAPI wglChoosePixelFormat(HDC hdc,
+GLAPI int GLWINAPI wglChoosePixelFormat(HDC hdc,
                                   CONST PIXELFORMATDESCRIPTOR *ppfd)
 {
     int		i,best = -1,bestdelta = 0x7FFFFFFF,delta,qt_valid_pix;
@@ -360,7 +447,7 @@ WINGDIAPI int WINAPI wglChoosePixelFormat(HDC hdc,
     return(best);
 }
 
-WINGDIAPI int WINAPI wglDescribePixelFormat(HDC hdc,int iPixelFormat,UINT nBytes,
+GLAPI int GLWINAPI wglDescribePixelFormat(HDC hdc,int iPixelFormat,UINT nBytes,
                                     LPPIXELFORMATDESCRIPTOR ppfd)
 {
     int		qt_valid_pix;
@@ -378,7 +465,7 @@ WINGDIAPI int WINAPI wglDescribePixelFormat(HDC hdc,int iPixelFormat,UINT nBytes
 /*
 * GetProcAddress - return the address of an appropriate extension
 */
-WINGDIAPI PROC WINAPI wglGetProcAddress(LPCSTR lpszProc)
+GLAPI PROC GLWINAPI wglGetProcAddress(LPCSTR lpszProc)
 {
     int		i;
     for(i = 0;i < qt_ext;i++)
@@ -389,7 +476,7 @@ WINGDIAPI PROC WINAPI wglGetProcAddress(LPCSTR lpszProc)
         return(NULL);
 }
 
-WINGDIAPI int WINAPI wglGetPixelFormat(HDC hdc)
+GLAPI int GLWINAPI wglGetPixelFormat(HDC hdc)
 {
     if(curPFD == 0)
     {
@@ -399,7 +486,7 @@ WINGDIAPI int WINAPI wglGetPixelFormat(HDC hdc)
     return(curPFD);
 }
 
-WINGDIAPI BOOL WINAPI wglSetPixelFormat(HDC hdc,int iPixelFormat,
+GLAPI BOOL GLWINAPI wglSetPixelFormat(HDC hdc,int iPixelFormat,
                                 PIXELFORMATDESCRIPTOR *ppfd)
 {
     int		qt_valid_pix;
@@ -414,7 +501,7 @@ WINGDIAPI BOOL WINAPI wglSetPixelFormat(HDC hdc,int iPixelFormat,
     return(TRUE);
 }
 
-WINGDIAPI BOOL WINAPI wglSwapBuffers(HDC hdc)
+GLAPI BOOL GLWINAPI wglSwapBuffers(HDC hdc)
 {
    if (ctx_current < 0)
       return FALSE;
