@@ -1,69 +1,20 @@
-/* $Id: wmesa.c,v 1.3 2000/03/03 23:21:57 brianp Exp $ */
+/* $Id: wmesa.c,v 1.17 2001/05/21 14:46:33 brianp Exp $ */
 
 /*
-*   File name   :   wmesa.c
-*  Version      :   2.3
-*
-*  Display driver for Mesa 2.3  under
-*   Windows95 and WindowsNT
-*
-*   Copyright (C) 1996-  Li Wei
-*  Address      :       Institute of Artificial Intelligence
-*               :           & Robotics
-*               :       Xi'an Jiaotong University
-*  Email        :       liwei@aiar.xjtu.edu.cn
-*  Web page :       http://sun.aiar.xjtu.edu.cn
-*
-*  This file and its associations are partially borrowed from the
-*  Windows NT driver for Mesa 1.8 , written by Mark Leaming
-*  (mark@rsinc.com).
-*/
-
-
-/*
- * $Log: wmesa.c,v $
- * Revision 1.3  2000/03/03 23:21:57  brianp
- * removed obsolete logicop function
+ * Windows (Win32) device driver for Mesa 3.4
  *
- * Revision 1.2  2000/02/17 20:52:02  brianp
- * replaced renderer_string() with get_string() func
+ * Original author:
  *
- * Revision 1.1.1.1  1999/08/19 00:55:42  jtg
- * Imported sources
- *
- * Revision 3.10  1999/06/15 01:35:06  brianp
- * small change to wmSetPixel() from TWILMOT@cpr.fr
- *
- * Revision 3.9  1999/05/11 19:06:01  brianp
- * fixed a few VB->Index bugs (mikec@ensoniq.com)
- *
- * Revision 3.8  1999/05/08 15:15:23  brianp
- * various updates from mikec@ensoniq.com
- *
- * Revision 3.7  1999/04/01 01:27:34  brianp
- * always flip Y coord in read_rgba_span()
- *
- * Revision 3.6  1999/03/28 21:17:27  brianp
- * updated SetBuffer driver function
- *
- * Revision 3.5  1999/03/16 01:36:42  brianp
- * patched dither() to check if Current is NULL, per xzhou@nyx.net
- *
- * Revision 3.4  1999/02/25 14:12:33  keithw
- * Merged in kw3 patch
- *
- * Revision 3.3  1999/01/03 03:08:57  brianp
- * Ted Jump's changes
- *
- * Revision 3.2  1998/08/29 00:26:01
- * updated for Mesa 3.0 to accomodate EGCS-Mingw32 build
- *
- * Revision 3.1  1998/06/11 01:42:08  brianp
- * updated for Mesa 3.0 device driver interface (but not tested)
- *
- * Revision 3.0  1998/06/11 01:18:25  brianp
- * initial revision
+ *  Copyright (C) 1996-  Li Wei
+ *  Address      :       Institute of Artificial Intelligence
+ *               :           & Robotics
+ *               :       Xi'an Jiaotong University
+ *  Email        :       liwei@aiar.xjtu.edu.cn
+ *  Web page :       http://sun.aiar.xjtu.edu.cn
  *
+ *  This file and its associations are partially borrowed from the
+ *  Windows NT driver for Mesa 1.8 , written by Mark Leaming
+ *  (mark@rsinc.com).
  */
 
 
@@ -84,7 +35,7 @@
 #include "depth.h"
 #include "wmesadef.h"
 
-#pragma warning ( disable : 4133 4761 )
+#pragma warning ( disable : 4100 4133 4761 )
 
 #ifdef PROFILE
 //  #include "profile.h"
@@ -415,10 +366,10 @@ static void clear_index(GLcontext* ctx, GLuint index)
 /*
 * Set the color used to clear the color buffer.
 */
-static void clear_color( GLcontext* ctx, GLubyte r, GLubyte g, GLubyte b, GLubyte a )
+static void clear_color( GLcontext* ctx, const GLchan color[4] )
 {
     STARTPROFILE
-        Current->clearpixel=RGB(r, g, b );
+        Current->clearpixel = RGB(color[0], color[1], color[2]);
     ENDPROFILE(clear_color)
 }
 
@@ -430,9 +381,9 @@ static void clear_color( GLcontext* ctx, GLubyte r, GLubyte g, GLubyte b, GLubyt
 */
 //static void clear(GLcontext* ctx,
 //                  GLboolean all,GLint x, GLint y, GLint width, GLint height )
-// TODO: I modified this function to match the prototype in dd.h. (swansma@geocities.com)
-//       dd.h does not explain what the return type is so I could not set this to the proper
-//       value.
+// TODO: I modified this function to match the prototype in
+//       dd.h. (swansma@geocities.com)
+
 static GLbitfield clear(GLcontext* ctx, GLbitfield mask,
                   GLboolean all, GLint x, GLint y, GLint width, GLint height)
 {
@@ -537,69 +488,31 @@ static GLbitfield clear(GLcontext* ctx, GLbitfield mask,
 
 
 
-/* Set the current color index. */
-static void set_index(GLcontext* ctx, GLuint index)
-{
-    STARTPROFILE
-        Current->pixel=index;
-    ENDPROFILE(set_index)
-}
-
-
-
-/* Set the current RGBA color. */
-static void set_color( GLcontext* ctx, GLubyte r, GLubyte g, GLubyte b, GLubyte a )
-{
-    STARTPROFILE
-        Current->pixel = RGB( r, g, b );
-    ENDPROFILE(set_color)
-}
-
-
-
-/* Set the index mode bitplane mask. */
-static GLboolean index_mask(GLcontext* ctx, GLuint mask)
-{
-    /* can't implement */
-    return GL_FALSE;
-}
-
-
-
-/* Set the RGBA drawing mask. */
-static GLboolean color_mask( GLcontext* ctx,
-                            GLboolean rmask, GLboolean gmask,
-                            GLboolean bmask, GLboolean amask)
-{
-    /* can't implement */
-    return GL_FALSE;
-}
-
-
-
-static void dither( GLcontext* ctx, GLboolean enable )
+static void enable( GLcontext* ctx, GLenum pname, GLboolean enable )
 {
    if (!Current)
       return;
 
-    if(enable == GL_FALSE){
-        Current->dither_flag = GL_FALSE;
-        if(Current->cColorBits == 8)
+   if (pname == GL_DITHER) {
+      if(enable == GL_FALSE){
+         Current->dither_flag = GL_FALSE;
+         if(Current->cColorBits == 8)
             Current->pixelformat = PF_INDEX8;
-    }
-    else{
-        if (Current->rgb_flag && Current->cColorBits == 8){
+      }
+      else{
+         if (Current->rgb_flag && Current->cColorBits == 8){
             Current->pixelformat = PF_DITHER8;
             Current->dither_flag = GL_TRUE;
-        }
-        else
+         }
+         else
             Current->dither_flag = GL_FALSE;
-    }
+      }
+   }
 }
 
 
 
-static GLboolean set_buffer( GLcontext* ctx, GLenum mode )
+static GLboolean set_draw_buffer( GLcontext* ctx, GLenum mode )
 {
    STARTPROFILE
    /* TODO: this could be better */
@@ -609,7 +522,15 @@ static GLboolean set_buffer( GLcontext* ctx, GLenum mode )
    else {
       return GL_FALSE;
    }
-   ENDPROFILE(set_buffer)
+   ENDPROFILE(set_draw_buffer)
+}
+
+
+static void set_read_buffer(GLcontext *ctx, GLframebuffer *colorBuffer,
+                            GLenum buffer )
+{
+   /* XXX todo */
+   return;
 }
 
 
@@ -710,8 +631,8 @@ static void fast_rgb_points( GLcontext* ctx, GLuint first, GLuint last )
 extern points_func choose_points_function( GLcontext* ctx )
 {
     STARTPROFILE
-        if (ctx->Point.Size==1.0 && !ctx->Point.SmoothFlag && ctx->RasterMask==0
-            && !ctx->Texture.Enabled  && ctx->Visual->RGBAflag) {
+        if (ctx->Point.Size==1.0 && !ctx->Point.SmoothFlag && ctx->_RasterMask==0
+            && !ctx->Texture._ReallyEnabled  && ctx->Visual->RGBAflag) {
             ENDPROFILE(choose_points_function)
                 return fast_rgb_points;
         }
@@ -768,8 +689,8 @@ static line_func choose_line_function( GLcontext* ctx )
 {
     STARTPROFILE
     if (ctx->Line.Width==1.0 && !ctx->Line.SmoothFlag && !ctx->Line.StippleFlag
-        && ctx->Light.ShadeModel==GL_FLAT && ctx->RasterMask==0
-        && !ctx->Texture.Enabled && Current->rgb_flag) {
+        && ctx->Light.ShadeModel==GL_FLAT && ctx->_RasterMask==0
+        && !ctx->Texture._ReallyEnabled && Current->rgb_flag) {
        ENDPROFILE(choose_line_function)
        return fast_flat_rgb_line;
     }
@@ -826,7 +747,7 @@ static void write_ci8_span( const GLcontext* ctx,
 */
 static void write_mono_ci_span(const GLcontext* ctx,
                                GLuint n,GLint x,GLint y,
-                               const GLubyte mask[])
+                               GLuint colorIndex, const GLubyte mask[])
 {
    STARTPROFILE
    GLuint i;
@@ -834,7 +755,7 @@ static void write_mono_ci_span(const GLcontext* ctx,
    assert(Current->rgb_flag==GL_FALSE);
    for (i=0; i<n; i++)
       if (mask[i])
-         Mem[i]=Current->pixel;
+         Mem[i]=colorIndex;
    ENDPROFILE(write_mono_ci_span)
 }
 
@@ -934,8 +855,9 @@ static void write_rgb_span( const GLcontext* ctx,
 */
 static void write_mono_rgba_span( const GLcontext* ctx,
                                   GLuint n, GLint x, GLint y,
-                                  const GLubyte mask[])
+                                  const GLchan color[4], const GLubyte mask[])
 {
+    ULONG pixel =  RGB( color[RCOMP], color[GCOMP], color[BCOMP] );
     STARTPROFILE
     GLuint i;
     HDC DC=DD_GETDC;
@@ -946,12 +868,12 @@ static void write_mono_rgba_span( const GLcontext* ctx,
         for (i=0; i<n; i++)
             if (mask[i])
                 // Trying
-                wmSetPixel(pwc,y,x+i,GetRValue(Current->pixel), GetGValue(Current->pixel), GetBValue(Current->pixel));
+                wmSetPixel(pwc,y,x+i,color[RCOMP], color[GCOMP], color[BCOMP]);
     }
     else {
         for (i=0; i<n; i++)
             if (mask[i])
-                SetPixel(DC, y, x+i, Current->pixel);
+                SetPixel(DC, y, x+i, pixel);
     }
     DD_RELEASEDC;
     ENDPROFILE(write_mono_rgba_span)
@@ -990,7 +912,7 @@ static void write_ci32_pixels( const GLcontext* ctx,
 static void write_mono_ci_pixels( const GLcontext* ctx,
                                   GLuint n,
                                   const GLint x[], const GLint y[],
-                                  const GLubyte mask[] )
+                                  GLuint colorIndex, const GLubyte mask[] )
 {
    STARTPROFILE
    GLuint i;
@@ -998,7 +920,7 @@ static void write_mono_ci_pixels( const GLcontext* ctx,
    for (i=0; i<n; i++) {
       if (mask[i]) {
          BYTE *Mem=Current->ScreenMem+FLIP(y[i])*Current->ScanWidth+x[i];
-         *Mem = Current->pixel;
+         *Mem = colorIndex;
       }
    }
    ENDPROFILE(write_mono_ci_pixels)
@@ -1018,7 +940,8 @@ static void write_rgba_pixels( const GLcontext* ctx,
     assert(Current->rgb_flag==GL_TRUE);
     for (i=0; i<n; i++)
        if (mask[i])
-          wmSetPixel(pwc, FLIP(y[i]),x[i],rgba[i][RCOMP],rgba[i][GCOMP],rgba[i][BCOMP]);
+          wmSetPixel(pwc, FLIP(y[i]), x[i],
+                     rgba[i][RCOMP], rgba[i][GCOMP], rgba[i][BCOMP]);
     DD_RELEASEDC;
     ENDPROFILE(write_rgba_pixels)
 }
@@ -1032,6 +955,7 @@ static void write_rgba_pixels( const GLcontext* ctx,
 static void write_mono_rgba_pixels( const GLcontext* ctx,
                                     GLuint n,
                                     const GLint x[], const GLint y[],
+                                    const GLchan color[4],
                                     const GLubyte mask[] )
 {
     STARTPROFILE
@@ -1041,8 +965,8 @@ static void write_mono_rgba_pixels( const GLcontext* ctx,
     assert(Current->rgb_flag==GL_TRUE);
     for (i=0; i<n; i++)
         if (mask[i])
-            wmSetPixel(pwc, FLIP(y[i]),x[i],GetRValue(Current->pixel),
-                       GetGValue(Current->pixel), GetBValue(Current->pixel));
+            wmSetPixel(pwc, FLIP(y[i]),x[i],color[RCOMP],
+                       color[GCOMP], color[BCOMP]);
     DD_RELEASEDC;
     ENDPROFILE(write_mono_rgba_pixels)
 }
@@ -1126,7 +1050,8 @@ static void read_rgba_pixels( const GLcontext* ctx,
    assert(Current->rgb_flag==GL_TRUE);
    for (i=0; i<n; i++) {
       if (mask[i]) {
-         Color=GetPixel(DC,x[i],FLIP(y[i]));
+         GLint y2 = Current->height - y[i] - 1;
+         Color=GetPixel(DC,x[i],y2);
          rgba[i][RCOMP] = GetRValue(Color);
          rgba[i][GCOMP] = GetGValue(Color);
          rgba[i][BCOMP] = GetBValue(Color);
@@ -1169,14 +1094,10 @@ void setup_DD_pointers( GLcontext* ctx )
     ctx->Driver.ClearColor = clear_color;
     ctx->Driver.Clear = clear;
 
-    ctx->Driver.Index = set_index;
-    ctx->Driver.Color = set_color;
-    ctx->Driver.IndexMask = index_mask;
-    ctx->Driver.ColorMask = color_mask;
+    ctx->Driver.Enable = enable;
 
-    ctx->Driver.Dither = dither;
-
-    ctx->Driver.SetBuffer = set_buffer;
+    ctx->Driver.SetDrawBuffer = set_draw_buffer;
+    ctx->Driver.SetReadBuffer = set_read_buffer;
     ctx->Driver.GetBufferSize = buffer_size;
 
     ctx->Driver.PointsFunc = choose_points_function(ctx);
@@ -1341,33 +1262,39 @@ WMesaContext WMesaCreateContext( HWND hWnd, HPALETTE* Pal,
 #endif
 
 
-    c->gl_visual = gl_create_visual(rgb_flag,
-                                    GL_FALSE,   /* software alpha */
-                                    db_flag,    /* db_flag */
-                                    GL_FALSE,   /* stereo */
-                                    16,         /* depth_bits */
-                                    8,          /* stencil_bits */
-                                    8,          /* accum_bits */
-                                    0,          /* index bits */
-                                    8,8,8,8 );  /* r, g, b, a bits */
+    c->gl_visual = _mesa_create_visual(rgb_flag,
+                                       db_flag,    /* db_flag */
+                                       GL_FALSE,   /* stereo */
+                                       8,8,8,8,    /* r, g, b, a bits */
+                                       0,          /* index bits */
+                                       16,         /* depth_bits */
+                                       8,          /* stencil_bits */
+                                       16,16,16,16,/* accum_bits */
+                                       1);
 
     if (!c->gl_visual) {
         return NULL;
     }
 
     /* allocate a new Mesa context */
-    c->gl_ctx = gl_create_context( c->gl_visual, NULL, c, GL_TRUE);
+    c->gl_ctx = _mesa_create_context( c->gl_visual, NULL, c, GL_TRUE);
 
     if (!c->gl_ctx) {
-        gl_destroy_visual( c->gl_visual );
+        _mesa_destroy_visual( c->gl_visual );
         free(c);
         return NULL;
     }
 
-    c->gl_buffer = gl_create_framebuffer( c->gl_visual );
+    _mesa_enable_sw_extensions(c->gl_ctx);
+
+    c->gl_buffer = _mesa_create_framebuffer( c->gl_visual,
+                                          c->gl_visual->DepthBits > 0,
+                                          c->gl_visual->StencilBits > 0,
+                                          c->gl_visual->AccumRedBits > 0,
+                                          c->gl_visual->AlphaBits > 0 );
     if (!c->gl_buffer) {
-        gl_destroy_visual( c->gl_visual );
-        gl_destroy_context( c->gl_ctx );
+        _mesa_destroy_visual( c->gl_visual );
+        _mesa_destroy_context( c->gl_ctx );
         free(c);
         return NULL;
     }
@@ -1386,9 +1313,9 @@ void WMesaDestroyContext( void )
     WC = c;
     if(c->hPalHalfTone != NULL)
         DeleteObject(c->hPalHalfTone);
-    gl_destroy_visual( c->gl_visual );
-    gl_destroy_framebuffer( c->gl_buffer );
-    gl_destroy_context( c->gl_ctx );
+    _mesa_destroy_visual( c->gl_visual );
+    _mesa_destroy_framebuffer( c->gl_buffer );
+    _mesa_destroy_context( c->gl_ctx );
 
     if (c->db_flag)
 #ifdef DDRAW
@@ -1424,12 +1351,12 @@ void /*APIENTRY*/ WMesaMakeCurrent( WMesaContext c )
         return;
 
     //gl_set_context( c->gl_ctx );
-    gl_make_current(c->gl_ctx, c->gl_buffer);
+    _mesa_make_current(c->gl_ctx, c->gl_buffer);
     setup_DD_pointers(c->gl_ctx);
     Current = c;
     if (Current->gl_ctx->Viewport.Width==0) {
         /* initialize viewport to window size */
-        gl_Viewport( Current->gl_ctx,
+        _mesa_set_viewport( Current->gl_ctx,
             0, 0, Current->width, Current->height );
     }
     if ((c->cColorBits <= 8 ) && (c->rgb_flag == GL_TRUE)){
@@ -1442,6 +1369,14 @@ void /*APIENTRY*/ WMesaMakeCurrent( WMesaContext c )
 void /*APIENTRY*/ WMesaSwapBuffers( void )
 {
     HDC DC = Current->hDC;
+    GET_CURRENT_CONTEXT(ctx);
+
+    /* If we're swapping the buffer associated with the current context
+     * we have to flush any pending rendering commands first.
+    */
+    if (Current && Current->gl_ctx == ctx)
+       _mesa_swapbuffers(ctx);
+
     if (Current->db_flag)
         wmFlush(Current);
 }
@@ -2153,6 +2088,7 @@ static void smooth_8A8B8G8R_z_triangle( GLcontext *ctx,
 {
     WMesaContext wmesa = (WMesaContext) ctx->DriverCtx;
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define INTERP_RGB 1
 #define PIXEL_ADDRESS(X,Y) PIXELADDR4(X,Y)
 #define PIXEL_TYPE GLuint
@@ -2907,11 +2843,11 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
     int depth = wmesa->cColorBits;
 
     if (ctx->Polygon.SmoothFlag)     return NULL;
-    if (ctx->Texture.Enabled)        return NULL;
+    if (ctx->Texture._ReallyEnabled)  return NULL;
     if (!wmesa->db_flag) return NULL;
     /*if (wmesa->xm_buffer->buffer==XIMAGE)*/ {
     if (   ctx->Light.ShadeModel==GL_SMOOTH
-        && ctx->RasterMask==DEPTH_BIT
+        && ctx->_RasterMask==DEPTH_BIT
         && ctx->Depth.Func==GL_LESS
         && ctx->Depth.Mask==GL_TRUE
         && ctx->Polygon.StippleFlag==GL_FALSE) {
@@ -2931,7 +2867,7 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
         }
     }
     if (   ctx->Light.ShadeModel==GL_FLAT
-        && ctx->RasterMask==DEPTH_BIT
+        && ctx->_RasterMask==DEPTH_BIT
         && ctx->Depth.Func==GL_LESS
         && ctx->Depth.Mask==GL_TRUE
         && ctx->Polygon.StippleFlag==GL_FALSE) {
@@ -2950,7 +2886,7 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
             return NULL;
         }
     }
-    if (   ctx->RasterMask==0   /* no depth test */
+    if (   ctx->_RasterMask==0   /* no depth test */
         && ctx->Light.ShadeModel==GL_SMOOTH
         && ctx->Polygon.StippleFlag==GL_FALSE) {
         switch (wmesa->pixelformat) {
@@ -2969,7 +2905,7 @@ static triangle_func choose_triangle_function( GLcontext *ctx )
         }
     }
 
-    if (   ctx->RasterMask==0   /* no depth test */
+    if (   ctx->_RasterMask==0   /* no depth test */
         && ctx->Light.ShadeModel==GL_FLAT
         && ctx->Polygon.StippleFlag==GL_FALSE) {
         switch (wmesa->pixelformat) {
