@@ -1,21 +1,21 @@
-/* $Id: hash.c,v 1.9.4.1 2000/11/26 21:10:26 brianp Exp $ */
+/* $Id: hash.c,v 1.10 2001/03/12 00:48:38 gareth Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
- * 
- * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
- * 
+ * Version:  3.5
+ *
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
@@ -101,6 +101,7 @@ void *_mesa_HashLookup(const struct _mesa_HashTable *table, GLuint key)
    const struct HashEntry *entry;
 
    assert(table);
+   assert(key);
 
    pos = key & (TABLE_SIZE-1);
    entry = table->Table[pos];
@@ -129,6 +130,7 @@ void _mesa_HashInsert(struct _mesa_HashTable *table, GLuint key, void *data)
    struct HashEntry *entry;
 
    assert(table);
+   assert(key);
 
    _glthread_LOCK_MUTEX(table->Mutex);
 
@@ -170,6 +172,7 @@ void _mesa_HashRemove(struct _mesa_HashTable *table, GLuint key)
    struct HashEntry *entry, *prev;
 
    assert(table);
+   assert(key);
 
    _glthread_LOCK_MUTEX(table->Mutex);
 
