@@ -1,21 +1,21 @@
-/* $Id: mmath.c,v 1.4 1999/11/08 07:36:44 brianp Exp $ */
+/* $Id: mmath.c,v 1.9 2001/03/12 00:48:38 gareth Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.1
- * 
- * Copyright (C) 1999  Brian Paul   All Rights Reserved.
- * 
+ * Version:  3.5
+ *
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
@@ -23,28 +23,18 @@
  * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/lib/GL/mesa/src/mmath.c,v 1.2 1999/04/04 00:20:28 dawes Exp $ */
-
-
-
 
 
 #ifdef PC_HEADER
 #include "all.h"
 #else
-#ifndef XFree86Server
-#include <math.h>
-#else
-#include "GL/xf86glx.h"
-#endif
-#include "GL/gl.h"
+#include "glheader.h"
 #include "mmath.h"
 #endif
 
 
 static int in_fast_math;
 
-
 /*
  * A High Speed, Low Precision Square Root
  * by Paul Lalonde and Robert Dawson
@@ -52,7 +42,7 @@ static int in_fast_math;
  */
 
 /*
- * SPARC implementation of a fast square root by table 
+ * SPARC implementation of a fast square root by table
  * lookup.
  * SPARC floating point format is as follows:
  *
@@ -99,8 +89,8 @@ static void init_sqrt(void)
       sqrttab[i+0x80] = (*fi & 0x7fffff) >> 16;
    }
 #else
-   (void) sqrttab;  /* silence compiler warning - unused var */
-#endif
+   (void) sqrttab;  /* silence compiler warnings */
+#endif /*FAST_MATH*/
 }
 
 
@@ -110,7 +100,7 @@ float gl_sqrt( float x )
    unsigned int *num = (unsigned int *)&x;
                                 /* to access the bits of a float in C
                                  * we must misuse pointers */
-                                                        
+
    short e;                     /* the exponent */
    if (x == 0.0F) return 0.0F;  /* check for square root of 0 */
    e = (*num >> 23) - 127;      /* get the exponent - on a SPARC the */
@@ -135,31 +125,59 @@ float gl_sqrt( float x )
 #endif
 }
 
-float gl_ubyte_to_float_color_tab[256];
-float gl_ubyte_to_float_255_color_tab[256];
 
-static void
-init_ubyte_color_tab(void)
-{
-   int i;
-   for (i = 0 ; i < 256 ; i++) {
-      gl_ubyte_to_float_color_tab[i] = (float) i * (1.0/255.0);
-      gl_ubyte_to_float_255_color_tab[i] = (float) i;
-   }
-}
+/* ubyte -> float conversion */
+float _mesa_ubyte_to_float_color_tab[256];
+
 
 /*
  * Initialize tables, etc for fast math functions.
  */
-void gl_init_math(void)
+void
+_mesa_init_math(void)
 {
    static GLboolean initialized = GL_FALSE;
 
    if (!initialized) {
+      int i;
+      for (i = 0; i < 256; i++) {
+         _mesa_ubyte_to_float_color_tab[i] = (float) i / 255.0;
+      }
+
       init_sqrt();
-      init_ubyte_color_tab();
 
       initialized = GL_TRUE;
       in_fast_math = 0;
+
+#if defined(_FPU_GETCW) && defined(_FPU_SETCW)
+      {
+         const char *debug = getenv("MESA_DEBUG");
+         if (debug && strcmp(debug, "FP")==0) {
+            /* die on FP exceptions */
+            fpu_control_t mask;
+            _FPU_GETCW(mask);
+            mask &= ~(_FPU_MASK_IM | _FPU_MASK_DM | _FPU_MASK_ZM
+                      | _FPU_MASK_OM | _FPU_MASK_UM);
+            _FPU_SETCW(mask);
+         }
+      }
+#endif
    }
 }
+
+
+
+/*
+ * Return number of bits set in given GLuint.
+ */
+GLuint
+_mesa_bitcount(GLuint n)
+{
+   GLuint bits;
+   for (bits = 0; n > 0; n = n >> 1) {
+      if (n & 1) {
+         bits++;
+      }
+   }
+   return bits;
+}
