@@ -1,10 +1,10 @@
-/* $Id: texutil.c,v 1.5.4.7 2001/05/14 17:54:25 brianp Exp $ */
+/* $Id: texutil.c,v 1.25 2001/06/15 15:22:08 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.4
+ * Version:  3.5
  *
- * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -34,21 +34,23 @@
 #include "context.h"
 #include "enums.h"
 #include "image.h"
+#include "macros.h"
 #include "mem.h"
+#include "mtypes.h"
 #include "texformat.h"
 #include "texutil.h"
-#include "types.h"
 #endif
 
-#define DBG 0
+#define DEBUG_TEXUTIL 0
 
 
 struct gl_texture_convert {
    GLint xoffset, yoffset, zoffset;	/* Subimage offset */
    GLint width, height, depth;		/* Subimage region */
 
-   GLint imageWidth, imageHeight;	/* Full image dimensions */
-   GLenum format, type;
+   GLint dstImageWidth, dstImageHeight;	/* Dest image size */
+                                        /* Needed for subimage replacement */
+   GLenum format, type;                 /* Source (user) format and type */
 
    const struct gl_pixelstore_attrib *packing;
 
@@ -59,35 +61,21 @@ struct gl_texture_convert {
 };
 
 typedef GLboolean (*convert_func)( struct gl_texture_convert *convert );
-typedef void (*unconvert_func)( struct gl_texture_convert *convert );
 
 #define CONVERT_STRIDE_BIT	0x1
 #define CONVERT_PACKING_BIT	0x2
 
-#ifdef __GNUC__
-#define ERROR_STRING	__FUNCTION__
-#else
-#define ERROR_STRING	__FILE__ ":" __LINE__
-#endif
-#define UNCONVERT_ERROR( format )					\
-do {									\
-   static char buffer[128];						\
-   sprintf( buffer, "illegal format %s in " ERROR_STRING "\n",		\
-	    gl_lookup_enum_by_nr( format ) );				\
-   gl_problem( NULL, buffer );						\
-} while (0)
 
 
-
-/* ================================================================
+/* =============================================================
  * RGBA8888 textures:
  */
 
 #define DST_TYPE		GLuint
 #define DST_TEXELS_PER_DWORD	1
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[3], src[2], src[1], src[0] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( src[3], src[2], src[1], src[0] )
 
 #define CONVERT_DIRECT
 
@@ -98,10 +86,10 @@ do {									\
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[0], src[1], src[2], src[3] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( src[0], src[1], src[2], src[3] )
 
-#define CONVERT_TEXEL_DWORD( src )	CONVERT_TEXEL( src )
+#define CONVERT_TEXEL_DWORD( dst, src )		CONVERT_TEXEL( dst, src )
 
 #define SRC_TEXEL_BYTES		4
 
@@ -110,10 +98,10 @@ do {									\
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[0], src[1], src[2], 0xff )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( src[0], src[1], src[2], 0xff )
 
-#define CONVERT_TEXEL_DWORD( src )	CONVERT_TEXEL( src )
+#define CONVERT_TEXEL_DWORD( dst, src )		CONVERT_TEXEL( dst, src )
 
 #define SRC_TEXEL_BYTES		3
 
@@ -157,140 +145,16 @@ CONVERT_RGBA8888( texsubimage2d )
 CONVERT_RGBA8888( texsubimage3d )
 
 
-static void
-unconvert_teximage_rgba8888( struct gl_texture_convert *convert )
-{
-   const GLubyte *src = (const GLubyte *)convert->srcImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA: {
-      GLuint *dst = (GLuint *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = PACK_COLOR_8888( src[0], src[1], src[2], src[3] );
-	 src += 4;
-      }
-      break;
-   }
-   case GL_RGB: {
-      GLubyte *dst = (GLubyte *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = src[3];
-	 *dst++ = src[2];
-	 *dst++ = src[1];
-	 src += 4;
-      }
-      break;
-   }
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
- * ABGR8888 textures:
- */
-
-#define DST_TYPE		GLuint
-#define DST_TEXELS_PER_DWORD	1
-
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[3], src[2], src[1], src[0] )
-
-#define CONVERT_DIRECT
-
-#define SRC_TEXEL_BYTES		4
-
-#define TAG(x) x##_abgr8888_direct
-#define PRESERVE_DST_TYPE
-#include "texutil_tmp.h"
-
-
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( 0xff, src[2], src[1], src[0] )
-
-#define CONVERT_TEXEL_DWORD( src )	CONVERT_TEXEL( src )
-
-#define SRC_TEXEL_BYTES		3
-
-#define TAG(x) x##_bgr888_to_abgr8888
-#include "texutil_tmp.h"
-
-
-#define CONVERT_ABGR8888( name )					\
-static GLboolean							\
-convert_##name##_abgr8888( struct gl_texture_convert *convert )		\
-{									\
-   convert_func *tab;							\
-   GLint index = convert->index;					\
-									\
-   if ( convert->format == GL_RGBA &&					\
-	convert->type == GL_UNSIGNED_BYTE )				\
-   {									\
-      tab = name##_tab_abgr8888_direct;					\
-   }									\
-   else if ( convert->format == GL_RGB &&				\
-	     convert->type == GL_UNSIGNED_BYTE )			\
-   {									\
-      tab = name##_tab_bgr888_to_abgr8888;				\
-   }									\
-   else									\
-   {									\
-      /* Can't handle this source format/type combination */		\
-      return GL_FALSE;							\
-   }									\
-									\
-   return tab[index]( convert );					\
-}
-
-CONVERT_ABGR8888( texsubimage2d )
-CONVERT_ABGR8888( texsubimage3d )
-
-
-static void
-unconvert_teximage_abgr8888( struct gl_texture_convert *convert )
-{
-   const GLubyte *src = (const GLubyte *)convert->srcImage;
-   GLint texels, i;
-
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA:
-      MEMCPY( convert->dstImage, src, texels * 4 );
-      break;
-   case GL_RGB: {
-      GLubyte *dst = (GLubyte *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = src[0];
-	 *dst++ = src[1];
-	 *dst++ = src[2];
-	 src += 4;
-      }
-      break;
-   }
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * ARGB8888 textures:
  */
 
 #define DST_TYPE		GLuint
 #define DST_TEXELS_PER_DWORD	1
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[3], src[2], src[1], src[0] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( src[3], src[2], src[1], src[0] )
 
 #define CONVERT_DIRECT
 
@@ -301,10 +165,10 @@ unconvert_teximage_abgr8888( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( src[3], src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( src[3], src[0], src[1], src[2] )
 
-#define CONVERT_TEXEL_DWORD( src )	CONVERT_TEXEL( src )
+#define CONVERT_TEXEL_DWORD( dst, src )		CONVERT_TEXEL( dst, src )
 
 #define SRC_TEXEL_BYTES		4
 
@@ -313,10 +177,10 @@ unconvert_teximage_abgr8888( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_8888( 0xff, src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_8888( 0xff, src[0], src[1], src[2] )
 
-#define CONVERT_TEXEL_DWORD( src )	CONVERT_TEXEL( src )
+#define CONVERT_TEXEL_DWORD( dst, src )		CONVERT_TEXEL( dst, src )
 
 #define SRC_TEXEL_BYTES		3
 
@@ -359,42 +223,8 @@ CONVERT_ARGB8888( texsubimage2d )
 CONVERT_ARGB8888( texsubimage3d )
 
 
-static void
-unconvert_teximage_argb8888( struct gl_texture_convert *convert )
-{
-   const GLubyte *src = (const GLubyte *)convert->srcImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA: {
-      GLuint *dst = (GLuint *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = PACK_COLOR_8888( src[3], src[0], src[1], src[2] );
-	 src += 4;
-      }
-      break;
-   }
-   case GL_RGB: {
-      GLubyte *dst = (GLubyte *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = src[2];
-	 *dst++ = src[1];
-	 *dst++ = src[0];
-	 src += 4;
-      }
-      break;
-   }
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * RGB888 textures:
  */
 
@@ -415,52 +245,16 @@ convert_texsubimage3d_rgb888( struct gl_texture_convert *convert )
 }
 
 
-static void
-unconvert_teximage_rgb888( struct gl_texture_convert *convert )
-{
-   UNCONVERT_ERROR( convert->format );
-}
 
-
-
-/* ================================================================
- * BGR888 textures:
- */
-
-static GLboolean
-convert_texsubimage2d_bgr888( struct gl_texture_convert *convert )
-{
-   /* This is a placeholder for now...
-    */
-   return GL_FALSE;
-}
-
-static GLboolean
-convert_texsubimage3d_bgr888( struct gl_texture_convert *convert )
-{
-   /* This is a placeholder for now...
-    */
-   return GL_FALSE;
-}
-
-
-static void
-unconvert_teximage_bgr888( struct gl_texture_convert *convert )
-{
-   UNCONVERT_ERROR( convert->format );
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * RGB565 textures:
  */
 
 #define DST_TYPE		GLushort
 #define DST_TEXELS_PER_DWORD	2
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_565( src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_565( src[0], src[1], src[2] )
 
 #define CONVERT_DIRECT
 
@@ -471,12 +265,12 @@ unconvert_teximage_bgr888( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_565( src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_565( src[0], src[1], src[2] )
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_565( src[0], src[1], src[2] )) |		\
-		 (PACK_COLOR_565( src[3], src[4], src[5] ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_565( src[0], src[1], src[2] )) |		\
+	       (PACK_COLOR_565( src[3], src[4], src[5] ) << 16))
 
 #define SRC_TEXEL_BYTES		3
 
@@ -485,12 +279,12 @@ unconvert_teximage_bgr888( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_565( src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_565( src[0], src[1], src[2] )
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_565( src[0], src[1], src[2] )) |		\
-		 (PACK_COLOR_565( src[4], src[5], src[6] ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_565( src[0], src[1], src[2] )) |		\
+	       (PACK_COLOR_565( src[4], src[5], src[6] ) << 16))
 
 #define SRC_TEXEL_BYTES		4
 
@@ -533,50 +327,16 @@ CONVERT_RGB565( texsubimage2d )
 CONVERT_RGB565( texsubimage3d )
 
 
-static void
-unconvert_teximage_rgb565( struct gl_texture_convert *convert )
-{
-   const GLushort *src = (const GLushort *)convert->srcImage;
-   GLubyte *dst = (GLubyte *)convert->dstImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA:
-      for ( i = 0 ; i < texels ; i++ ) {
-	 GLushort s = *src++;
-	 *dst++ = ((s >> 8) & 0xf8) * 255 / 0xf8;
-	 *dst++ = ((s >> 3) & 0xfc) * 255 / 0xfc;
-	 *dst++ = ((s << 3) & 0xf8) * 255 / 0xf8;
-	 *dst++ = 0xff;
-      }
-      break;
-   case GL_RGB:
-      for ( i = 0 ; i < texels ; i++ ) {
-	 GLushort s = *src++;
-	 *dst++ = ((s >> 8) & 0xf8) * 255 / 0xf8;
-	 *dst++ = ((s >> 3) & 0xfc) * 255 / 0xfc;
-	 *dst++ = ((s << 3) & 0xf8) * 255 / 0xf8;
-      }
-      break;
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * ARGB4444 textures:
  */
 
 #define DST_TYPE		GLushort
 #define DST_TEXELS_PER_DWORD	2
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_4444( src[3], src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_4444( src[3], src[0], src[1], src[2] )
 
 #define CONVERT_DIRECT
 
@@ -587,16 +347,16 @@ unconvert_teximage_rgb565( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_4444( src[3], src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_4444( src[3], src[0], src[1], src[2] )
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_4444( src[3], src[0], src[1], src[2] )) |	\
-		 (PACK_COLOR_4444( src[7], src[4], src[5], src[6] ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_4444( src[3], src[0], src[1], src[2] )) |	\
+	       (PACK_COLOR_4444( src[7], src[4], src[5], src[6] ) << 16))
 
 #define SRC_TEXEL_BYTES		4
 
-#define TAG(x) x##_rgba8888_to_argb4444
+#define TAG(x) x##_abgr8888_to_argb4444
 #include "texutil_tmp.h"
 
 
@@ -615,7 +375,7 @@ convert_##name##_argb4444( struct gl_texture_convert *convert )		\
    else if ( convert->format == GL_RGBA &&				\
 	     convert->type == GL_UNSIGNED_BYTE )			\
    {									\
-      tab = name##_tab_rgba8888_to_argb4444;				\
+      tab = name##_tab_abgr8888_to_argb4444;				\
    }									\
    else									\
    {									\
@@ -630,42 +390,16 @@ CONVERT_ARGB4444( texsubimage2d )
 CONVERT_ARGB4444( texsubimage3d )
 
 
-static void
-unconvert_teximage_argb4444( struct gl_texture_convert *convert )
-{
-   const GLushort *src = (const GLushort *)convert->srcImage;
-   GLubyte *dst = (GLubyte *)convert->dstImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA:
-      for ( i = 0 ; i < texels ; i++ ) {
-	 GLushort s = *src++;
-	 *dst++ = ((s >>  8) & 0xf) * 255 / 0xf;
-	 *dst++ = ((s >>  4) & 0xf) * 255 / 0xf;
-	 *dst++ = ((s      ) & 0xf) * 255 / 0xf;
-	 *dst++ = ((s >> 12) & 0xf) * 255 / 0xf;
-      }
-      break;
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * ARGB1555 textures:
  */
 
 #define DST_TYPE		GLushort
 #define DST_TEXELS_PER_DWORD	2
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_1555( src[3], src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_1555( src[3], src[0], src[1], src[2] )
 
 #define CONVERT_DIRECT
 
@@ -676,16 +410,32 @@ unconvert_teximage_argb4444( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_1555( src[3], src[0], src[1], src[2] )
+#define CONVERT_TEXEL( dst, src )					\
+	{ const GLushort s = *(GLushort *)src;				\
+	  dst = (s >> 1) | ((s & 1) << 15); }
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_1555( src[3], src[0], src[1], src[2] )) |	\
-		 (PACK_COLOR_1555( src[7], src[4], src[5], src[6] ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	{ const GLuint s = ((fi_type *)src)->i;				\
+	  dst = (((s & 0xfffefffe) >> 1) |				\
+		 ((s & 0x00010001) << 15)); }
+
+#define SRC_TEXEL_BYTES		2
+
+#define TAG(x) x##_rgba5551_to_argb1555
+#define PRESERVE_DST_TYPE
+#include "texutil_tmp.h"
+
+
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_1555( src[3], src[0], src[1], src[2] )
+
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_1555( src[3], src[0], src[1], src[2] )) |	\
+	       (PACK_COLOR_1555( src[7], src[4], src[5], src[6] ) << 16))
 
 #define SRC_TEXEL_BYTES		4
 
-#define TAG(x) x##_rgba8888_to_argb1555
+#define TAG(x) x##_abgr8888_to_argb1555
 #include "texutil_tmp.h"
 
 
@@ -701,10 +451,15 @@ convert_##name##_argb1555( struct gl_texture_convert *convert )		\
    {									\
       tab = name##_tab_argb1555_direct;					\
    }									\
+   else if ( convert->format == GL_RGBA &&				\
+	     convert->type == GL_UNSIGNED_SHORT_5_5_5_1 )		\
+   {									\
+      tab = name##_tab_rgba5551_to_argb1555;				\
+   }									\
    else if ( convert->format == GL_RGBA &&				\
 	     convert->type == GL_UNSIGNED_BYTE )			\
    {									\
-      tab = name##_tab_rgba8888_to_argb1555;				\
+      tab = name##_tab_abgr8888_to_argb1555;				\
    }									\
    else									\
    {									\
@@ -719,42 +474,16 @@ CONVERT_ARGB1555( texsubimage2d )
 CONVERT_ARGB1555( texsubimage3d )
 
 
-static void
-unconvert_teximage_argb1555( struct gl_texture_convert *convert )
-{
-   const GLushort *src = (const GLushort *)convert->srcImage;
-   GLubyte *dst = (GLubyte *)convert->dstImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_RGBA:
-      for ( i = 0 ; i < texels ; i++ ) {
-	 GLushort s = *src++;
-	 *dst++ = ((s >> 10) & 0xf8) * 255 / 0xf8;
-	 *dst++ = ((s >>  5) & 0xf8) * 255 / 0xf8;
-	 *dst++ = ((s      ) & 0xf8) * 255 / 0xf8;
-	 *dst++ = ((s >> 15) & 0x01) * 255;
-      }
-      break;
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * AL88 textures:
  */
 
 #define DST_TYPE		GLushort
 #define DST_TEXELS_PER_DWORD	2
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_88( src[0], src[1] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_88( src[0], src[1] )
 
 #define CONVERT_DIRECT
 
@@ -765,12 +494,12 @@ unconvert_teximage_argb1555( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_88( src[0], 0x00 )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_88( src[0], 0x00 )
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_88( src[0], 0x00 )) |			\
-		 (PACK_COLOR_88( src[1], 0x00 ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_88( src[0], 0x00 )) |			\
+	       (PACK_COLOR_88( src[1], 0x00 ) << 16))
 
 #define SRC_TEXEL_BYTES		1
 
@@ -779,16 +508,30 @@ unconvert_teximage_argb1555( struct gl_texture_convert *convert )
 #include "texutil_tmp.h"
 
 
-#define CONVERT_TEXEL( src )						\
-		PACK_COLOR_88( 0xff, src[0] )
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_88( 0xff, src[0] )
 
-#define CONVERT_TEXEL_DWORD( src )					\
-		((PACK_COLOR_88( 0xff, src[0] )) |			\
-		 (PACK_COLOR_88( 0xff, src[1] ) << 16))
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_88( 0xff, src[0] )) |			\
+	       (PACK_COLOR_88( 0xff, src[1] ) << 16))
 
 #define SRC_TEXEL_BYTES		1
 
 #define TAG(x) x##_l8_to_al88
+#define PRESERVE_DST_TYPE
+#include "texutil_tmp.h"
+
+
+#define CONVERT_TEXEL( dst, src )					\
+	dst = PACK_COLOR_88( src[3], src[0] )
+
+#define CONVERT_TEXEL_DWORD( dst, src )					\
+	dst = ((PACK_COLOR_88( src[3], src[0] )) |			\
+	       (PACK_COLOR_88( src[7], src[1] ) << 16))
+
+#define SRC_TEXEL_BYTES		4
+
+#define TAG(x) x##_abgr8888_to_al88
 #include "texutil_tmp.h"
 
 
@@ -814,6 +557,11 @@ convert_##name##_al88( struct gl_texture_convert *convert )		\
    {									\
       tab = name##_tab_l8_to_al88;					\
    }									\
+   else if ( convert->format == GL_RGBA &&				\
+	     convert->type == GL_UNSIGNED_BYTE )			\
+   {									\
+      tab = name##_tab_abgr8888_to_al88;				\
+   }									\
    else									\
    {									\
       /* Can't handle this source format/type combination */		\
@@ -827,43 +575,8 @@ CONVERT_AL88( texsubimage2d )
 CONVERT_AL88( texsubimage3d )
 
 
-static void
-unconvert_teximage_al88( struct gl_texture_convert *convert )
-{
-   const GLubyte *src = (const GLubyte *)convert->srcImage;
-   GLint texels, i;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_LUMINANCE_ALPHA:
-      MEMCPY( convert->dstImage, src, texels * 2 );
-      break;
-   case GL_ALPHA: {
-      GLubyte *dst = (GLubyte *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = src[1];
-	 src += 2;
-      }
-      break;
-   }
-   case GL_LUMINANCE: {
-      GLubyte *dst = (GLubyte *)convert->dstImage;
-      for ( i = 0 ; i < texels ; i++ ) {
-	 *dst++ = src[0];
-	 src += 2;
-      }
-      break;
-   }
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * RGB332 textures:
  */
 
@@ -884,22 +597,15 @@ convert_texsubimage3d_rgb332( struct gl_texture_convert *convert )
 }
 
 
-static void
-unconvert_teximage_rgb332( struct gl_texture_convert *convert )
-{
-   UNCONVERT_ERROR( convert->format );
-}
 
-
-
-/* ================================================================
+/* =============================================================
  * CI8 (and all other single-byte texel) textures:
  */
 
 #define DST_TYPE		GLubyte
 #define DST_TEXELS_PER_DWORD	4
 
-#define CONVERT_TEXEL( src )	src[0]
+#define CONVERT_TEXEL( dst, src )	dst = src[0]
 
 #define CONVERT_DIRECT
 
@@ -937,39 +643,15 @@ CONVERT_CI8( texsubimage2d )
 CONVERT_CI8( texsubimage3d )
 
 
-static void
-unconvert_teximage_ci8( struct gl_texture_convert *convert )
-{
-   const GLubyte *src = (const GLubyte *)convert->srcImage;
-   GLint texels;
 
-   texels = convert->width * convert->height * convert->depth;
-
-   switch ( convert->format ) {
-   case GL_ALPHA:
-   case GL_LUMINANCE:
-   case GL_INTENSITY:
-   case GL_COLOR_INDEX:
-      MEMCPY( convert->dstImage, src, texels );
-      break;
-   default:
-      UNCONVERT_ERROR( convert->format );
-      break;
-   }
-}
-
-
-
-/* ================================================================
+/* =============================================================
  * Global entry points
  */
 
 static convert_func gl_convert_texsubimage2d_tab[] = {
    convert_texsubimage2d_rgba8888,
-   convert_texsubimage2d_abgr8888,
    convert_texsubimage2d_argb8888,
    convert_texsubimage2d_rgb888,
-   convert_texsubimage2d_bgr888,
    convert_texsubimage2d_rgb565,
    convert_texsubimage2d_argb4444,
    convert_texsubimage2d_argb1555,
@@ -983,10 +665,8 @@ static convert_func gl_convert_texsubimage2d_tab[] = {
 
 static convert_func gl_convert_texsubimage3d_tab[] = {
    convert_texsubimage3d_rgba8888,
-   convert_texsubimage3d_abgr8888,
    convert_texsubimage3d_argb8888,
    convert_texsubimage3d_rgb888,
-   convert_texsubimage3d_bgr888,
    convert_texsubimage3d_rgb565,
    convert_texsubimage3d_argb4444,
    convert_texsubimage3d_argb1555,
@@ -998,23 +678,6 @@ static convert_func gl_convert_texsubimage3d_tab[] = {
    convert_texsubimage3d_ci8,
 };
 
-static unconvert_func gl_unconvert_teximage_tab[] = {
-   unconvert_teximage_rgba8888,
-   unconvert_teximage_abgr8888,
-   unconvert_teximage_argb8888,
-   unconvert_teximage_rgb888,
-   unconvert_teximage_bgr888,
-   unconvert_teximage_rgb565,
-   unconvert_teximage_argb4444,
-   unconvert_teximage_argb1555,
-   unconvert_teximage_al88,
-   unconvert_teximage_rgb332,
-   unconvert_teximage_ci8,		/* These are all the same... */
-   unconvert_teximage_ci8,
-   unconvert_teximage_ci8,
-   unconvert_teximage_ci8,
-};
-
 
 /* See if we need to care about the pixel store attributes when we're
  * converting the texture image.  This should be stored as
@@ -1079,11 +742,37 @@ _mesa_convert_texsubimage1d( GLint mesaFormat,
    return gl_convert_texsubimage2d_tab[mesaFormat]( &convert );
 }
 
+
+/* Convert a user's 2D image into a texture image.  This basically
+ * repacks pixel data into the special texture formats used by core Mesa
+ * and the DRI drivers.  This function can do full images or subimages.
+ *
+ * We return a boolean because this function may not accept some kinds
+ * of source image formats and/or types.  For example, if the incoming
+ * format/type = GL_BGR, GL_UNSIGNED_INT this function probably won't
+ * be able to do the conversion.
+ *
+ * In that case, the incoming image should first be simplified to one of
+ * the "canonical" formats (GL_ALPHA, GL_LUMINANCE, GL_LUMINANCE_ALPHA,
+ * GL_INTENSITY, GL_RGB, GL_RGBA) and types (GL_CHAN).  We can do that
+ * with the _mesa_transfer_teximage() function.  That function will also
+ * do image transfer operations such as scale/bias and convolution.
+ *
+ * Input:
+ *   mesaFormat - one of the MESA_FORMAT_* values from texformat.h
+ *   xoffset, yoffset - position in dest image to put data
+ *   width, height - incoming image size, also size of dest region.
+ *   dstImageWidth - width (row stride) of dest image in pixels
+ *   format, type - incoming image format and type
+ *   packing - describes incoming image packing
+ *   srcImage - pointer to source image
+ *   destImage - pointer to dest image
+ */
 GLboolean
 _mesa_convert_texsubimage2d( GLint mesaFormat,
 			     GLint xoffset, GLint yoffset,
 			     GLint width, GLint height,
-			     GLint imageWidth,
+			     GLint destImageWidth,
 			     GLenum format, GLenum type,
 			     const struct gl_pixelstore_attrib *packing,
 			     const GLvoid *srcImage, GLvoid *dstImage )
@@ -1103,7 +792,7 @@ _mesa_convert_texsubimage2d( GLint mesaFormat,
    convert.yoffset = yoffset;
    convert.width = width;
    convert.height = height;
-   convert.imageWidth = imageWidth;
+   convert.dstImageWidth = destImageWidth;
    convert.format = format;
    convert.type = type;
    convert.packing = packing;
@@ -1115,7 +804,7 @@ _mesa_convert_texsubimage2d( GLint mesaFormat,
    if ( convert_needs_packing( packing, format, type ) )
       convert.index |= CONVERT_PACKING_BIT;
 
-   if ( width != imageWidth )
+   if ( width != destImageWidth )
       convert.index |= CONVERT_STRIDE_BIT;
 
    return gl_convert_texsubimage2d_tab[mesaFormat]( &convert );
@@ -1125,7 +814,7 @@ GLboolean
 _mesa_convert_texsubimage3d( GLint mesaFormat,
 			     GLint xoffset, GLint yoffset, GLint zoffset,
 			     GLint width, GLint height, GLint depth,
-			     GLint imageWidth, GLint imageHeight,
+			     GLint dstImageWidth, GLint dstImageHeight,
 			     GLenum format, GLenum type,
 			     const struct gl_pixelstore_attrib *packing,
 			     const GLvoid *srcImage, GLvoid *dstImage )
@@ -1147,8 +836,8 @@ _mesa_convert_texsubimage3d( GLint mesaFormat,
    convert.width = width;
    convert.height = height;
    convert.depth = depth;
-   convert.imageWidth = imageWidth;
-   convert.imageHeight = imageHeight;
+   convert.dstImageWidth = dstImageWidth;
+   convert.dstImageHeight = dstImageHeight;
    convert.format = format;
    convert.type = type;
    convert.packing = packing;
@@ -1160,7 +849,7 @@ _mesa_convert_texsubimage3d( GLint mesaFormat,
    if ( convert_needs_packing( packing, format, type ) )
       convert.index |= CONVERT_PACKING_BIT;
 
-   if ( width != imageWidth || height != imageHeight )
+   if ( width != dstImageWidth || height != dstImageHeight )
       convert.index |= CONVERT_STRIDE_BIT;
 
    return gl_convert_texsubimage3d_tab[mesaFormat]( &convert );
@@ -1168,97 +857,25 @@ _mesa_convert_texsubimage3d( GLint mesaFormat,
 
 
 
-void _mesa_unconvert_teximage1d( GLint mesaFormat, GLenum format, GLint width,
-				 const GLvoid *srcImage, GLvoid *dstImage )
-{
-   struct gl_texture_convert convert;
-
-   ASSERT( srcImage );
-   ASSERT( dstImage );
-
-   ASSERT( mesaFormat >= MESA_FORMAT_RGBA8888 );
-   ASSERT( mesaFormat <= MESA_FORMAT_CI8 );
-
-   /* Make it easier to pass all the parameters around.
-    */
-   convert.width = width;
-   convert.height = 1;
-   convert.depth = 1;
-   convert.format = format;
-   convert.srcImage = srcImage;
-   convert.dstImage = dstImage;
-
-   gl_unconvert_teximage_tab[mesaFormat]( &convert );
-}
-
-void _mesa_unconvert_teximage2d( GLint mesaFormat, GLenum format,
-				 GLint width, GLint height,
-				 const GLvoid *srcImage, GLvoid *dstImage )
-{
-   struct gl_texture_convert convert;
-
-   ASSERT( srcImage );
-   ASSERT( dstImage );
-
-   ASSERT( mesaFormat >= MESA_FORMAT_RGBA8888 );
-   ASSERT( mesaFormat <= MESA_FORMAT_CI8 );
-
-   /* Make it easier to pass all the parameters around.
-    */
-   convert.width = width;
-   convert.height = height;
-   convert.depth = 1;
-   convert.format = format;
-   convert.srcImage = srcImage;
-   convert.dstImage = dstImage;
-
-   gl_unconvert_teximage_tab[mesaFormat]( &convert );
-}
-
-void _mesa_unconvert_teximage3d( GLint mesaFormat, GLenum format,
-				 GLint width, GLint height, GLint depth,
-				 const GLvoid *srcImage, GLvoid *dstImage )
-{
-   struct gl_texture_convert convert;
-
-   ASSERT( srcImage );
-   ASSERT( dstImage );
-
-   ASSERT( mesaFormat >= MESA_FORMAT_RGBA8888 );
-   ASSERT( mesaFormat <= MESA_FORMAT_CI8 );
-
-   /* Make it easier to pass all the parameters around.
-    */
-   convert.width = width;
-   convert.height = height;
-   convert.depth = depth;
-   convert.format = format;
-   convert.srcImage = srcImage;
-   convert.dstImage = dstImage;
-
-   gl_unconvert_teximage_tab[mesaFormat]( &convert );
-}
-
-
-
 /* Nearest filtering only (for broken hardware that can't support
  * all aspect ratios).  This can be made a lot faster, but I don't
  * really care enough...
  */
-void _mesa_rescale_teximage2d( GLint texelBytes,
+void _mesa_rescale_teximage2d( GLuint bytesPerPixel, GLuint dstRowStride,
 			       GLint srcWidth, GLint srcHeight,
 			       GLint dstWidth, GLint dstHeight,
 			       const GLvoid *srcImage, GLvoid *dstImage )
 {
    GLint row, col;
 
-#define INNER_LOOP( HOP, WOP )						\
+#define INNER_LOOP( TYPE, HOP, WOP )					\
    for ( row = 0 ; row < dstHeight ; row++ ) {				\
       GLint srcRow = row HOP hScale;					\
       for ( col = 0 ; col < dstWidth ; col++ ) {			\
 	 GLint srcCol = col WOP wScale;					\
-	 *dst++ = src[srcRow * srcWidth + srcCol];			\
+	 dst[col] = src[srcRow * srcWidth + srcCol];			\
       }									\
+      dst = (TYPE *) ((GLubyte *) dst + dstRowStride);			\
    }									\
 
 #define RESCALE_IMAGE( TYPE )						\
@@ -1270,27 +887,27 @@ do {									\
       const GLint hScale = dstHeight / srcHeight;			\
       if ( srcWidth <= dstWidth ) {					\
 	 const GLint wScale = dstWidth / srcWidth;			\
-	 INNER_LOOP( /, / );						\
+	 INNER_LOOP( TYPE, /, / );					\
       }									\
       else {								\
 	 const GLint wScale = srcWidth / dstWidth;			\
-	 INNER_LOOP( /, * );						\
+	 INNER_LOOP( TYPE, /, * );					\
       }									\
    }									\
    else {								\
       const GLint hScale = srcHeight / dstHeight;			\
       if ( srcWidth <= dstWidth ) {					\
 	 const GLint wScale = dstWidth / srcWidth;			\
-	 INNER_LOOP( *, / );						\
+	 INNER_LOOP( TYPE, *, / );					\
       }									\
       else {								\
 	 const GLint wScale = srcWidth / dstWidth;			\
-	 INNER_LOOP( *, * );						\
+	 INNER_LOOP( TYPE, *, * );					\
       }									\
    }									\
 } while (0)
 
-   switch ( texelBytes ) {
+   switch ( bytesPerPixel ) {
    case 4:
       RESCALE_IMAGE( GLuint );
       break;
@@ -1302,5 +919,7 @@ do {									\
    case 1:
       RESCALE_IMAGE( GLubyte );
       break;
+   default:
+      _mesa_problem(NULL,"unexpected bytes/pixel in _mesa_rescale_teximage2d");
    }
 }
